from time import sleep
from genworlds.agents.abstracts.state_manager import AbstractStateManager
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.agents.abstracts.agent import AbstractAgent

from genworlds.worlds.concrete.base.actions import AgentWantsUpdatedStateEvent
from genworlds.agents.memories.simulation_memory import SimulationMemory


class BasicAssistantStateManager(AbstractStateManager):
    """This state manager keeps track of the current state of the agent."""

    def __init__(
        self, host_agent: AbstractAgent, state: AbstractAgentState, openai_api_key: str
    ):
        self.host_agent = host_agent
        self.state = state
        if not state:
            self.state = self._initialize_state(host_agent)
        else:
            self.state = state

        self.memory = SimulationMemory(openai_api_key=openai_api_key)

    def _initialize_state(
        self,
    ) -> (
        AbstractAgentState
    ):  # should trigger an action to get the initial state from the world
        return AbstractAgentState(
            name=self.host_agent.name,
            id=self.host_agent.id,
            goals=[
                "Starts waiting and sleeps till the user starts a new question.",
                f"Once {self.host_agent.name} receives a user's question, he makes sure to have all the information before sending the answer to the user.",
                f"When {self.host_agent.name} has all the required information, he speaks to the user with the results through the agent_speaks_with_user_event.",
                "After sending the response, he waits for the next user question.",
                "If you have been waiting for any object or entity to send you an event for over 30 seconds, you will wait and sleep until you receive a new event.",
            ],
            available_entities={},
            available_action_schemas={},
            current_action_chain=[],
            host_world_prompt="",
            is_asleep=False,
            simulation_memory_persistent_path="./",
            important_event_types=set(),  # fill
            interesting_event_types=set(),  # fill
            wakeup_event_types=set(),  # fill
            action_schema_chains=[],
        )

    def get_updated_state(self) -> AbstractAgentState:
        self.host_agent.send_event(
            AgentWantsUpdatedStateEvent(
                sender_id=self.host_agent.id, target_id=self.host_agent.host_world_id
            )
        )
        # retrieve memory and update last_retrieved_memory
        query = "No plan" if self.state.plan == [] else str(self.state.plan)
        self.host_agent.state_manager.state.last_retrieved_memory = (
            self.memory.get_event_stream_memories(query=query)
        )
        sleep(2)
        # meanwhile the concrete.base world processes the request and triggers the basic_assistant actions that update the state
        return self.state



import threading
import json
import time

import websocket
from colorama import Fore

from genworlds.utils.logging_factory import LoggingFactory


class SimulationSocketClient:
    """
    A client for managing connections to a simulation socket server.
    """

    def __init__(
        self,
        process_event,
        url: str = "ws://127.0.0.1:7456/ws",
        send_initial_event=None,
        reconnect_interval=5,
        log_level=None,
    ) -> None:
        self.url = url
        self.websocket = websocket.WebSocketApp(
            self.url,
            on_open=self.on_open,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
        )
        # Callback function to process events
        self.process_event = process_event
        self.send_initial_event = send_initial_event
        self.reconnect_interval = reconnect_interval
        self.log_level = log_level

    def on_open(self, ws):
        self.logger().info(f"Connected to world socket server {self.url}")
        if self.send_initial_event:
            self.send_initial_event()
            self.logger().debug(f"Initial event sent")

    def on_error(self, ws, error):
        self.logger().error("World socket client error", exc_info=error)

    def on_close(self, *args):
        self.logger().info("World socket client closed connection", args)
        if self.reconnect_interval:
            self.logger().info(
                f"Attempting to reconnect in {self.reconnect_interval} seconds"
            )
            time.sleep(self.reconnect_interval)
            self.logger().info("Attempting to reconnect")
            self.websocket.run_forever()

    def on_message(self, ws, message):
        self.logger().debug(f"Received: {message}")
        if self.process_event:
            self.process_event(json.loads(message))

    def send_message(self, message):
        self.websocket.send(message)
        self.logger().debug(f"Sent: {message}")

    def logger(self):
        return LoggingFactory.get_logger(
            threading.current_thread().name, level=self.log_level
        )



from typing import List
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.agents.concrete.basic_assistant.state_manager import (
    BasicAssistantStateManager,
)
from genworlds.agents.concrete.basic_assistant.action_planner import (
    BasicAssistantActionPlanner,
)
from genworlds.events.abstracts.action import AbstractAction
from genworlds.agents.concrete.basic_assistant.actions import (
    UpdateAgentAvailableEntities,
    UpdateAgentAvailableActionSchemas,
    AgentGoesToSleep,
    AgentListensEvents,
    AgentSpeaksWithUser,
    AgentSpeaksWithAgent,
)
from genworlds.agents.abstracts.thought import AbstractThought


class BasicAssistant(AbstractAgent):
    def __init__(
        self,
        openai_api_key: str,
        name: str,
        id: str,
        description: str,
        host_world_id: str = None,
        initial_agent_state: AbstractAgentState = None,
        action_classes: List[type[AbstractAction]] = [],
        other_thoughts: List[AbstractThought] = [],
    ):
        state_manager = BasicAssistantStateManager(
            self, initial_agent_state, openai_api_key
        )
        action_planner = BasicAssistantActionPlanner(
            openai_api_key=openai_api_key,
            initial_agent_state=state_manager.state,
            other_thoughts=other_thoughts,
            host_agent=self,
        )

        actions = []
        for action_class in action_classes:
            actions.append(action_class(host_object=self))

        actions.append(UpdateAgentAvailableEntities(host_object=self))
        actions.append(UpdateAgentAvailableActionSchemas(host_object=self))
        actions.append(AgentGoesToSleep(host_object=self))
        actions.append(AgentListensEvents(host_object=self))
        actions.append(AgentSpeaksWithUser(host_object=self))
        actions.append(AgentSpeaksWithAgent(host_object=self))

        super().__init__(
            name, id, description, state_manager, action_planner, host_world_id, actions
        )

    def add_wakeup_event(self, event_class: AbstractEvent):
        self.state_manager.state.wakeup_event_types.add(
            event_class.__fields__["event_type"].default
        )

    def add_memory_ignored_event(self, event_type: str):
        self.state_manager.state.memory_ignored_event_types.add(event_type)



from datetime import datetime
from typing import Dict, Any, Optional, Type
from pydantic import create_model, BaseModel

TYPE_MAPPING = {
    "string": (str, ...),
    "integer": (int, ...),
    "boolean": (bool, ...),
    "array": (list, ...),
    "object": (dict, ...),
    "number": (float, ...),
}


def json_schema_to_pydantic_model(schema: Dict[str, Any]) -> Type[BaseModel]:
    name = schema.get("title", "DynamicModel")
    required_fields = schema.get("required", [])

    fields = {}
    for k, v in schema["properties"].items():
        field_type, default_value = TYPE_MAPPING.get(v.get("type"), (Any, ...))

        # Special handling for date-time format
        if v.get("format") == "date-time":
            field_type = datetime

        # If the field is not required or has a default value, it's optional
        if k not in required_fields and "default" in v:
            default_value = v["default"]
            field_type = Optional[field_type]

        fields[k] = (field_type, default_value)

    return create_model(name, **fields)



from abc import ABC

from typing import Optional
from pydantic import BaseModel, Field
from datetime import datetime


class AbstractEvent(ABC, BaseModel):
    event_type: str
    description: str
    summary: Optional[str]
    created_at: datetime = Field(default_factory=datetime.now)
    sender_id: str
    target_id: Optional[str]



from typing import Type
import json
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.agents.abstracts.thought import AbstractThought
from langchain.chat_models import ChatOpenAI
from langchain.chains.openai_functions import (
    create_structured_output_chain,
)
from langchain.chat_models import ChatOpenAI
from langchain.prompts import ChatPromptTemplate


class EventFillerThought(AbstractThought):
    def __init__(
        self,
        agent_state: AbstractAgentState,
        openai_api_key: str,
        model_name: str = "gpt-3.5-turbo",
    ):
        self.agent_state = agent_state
        self.model_name = model_name
        self.llm = ChatOpenAI(
            model=self.model_name, openai_api_key=openai_api_key, temperature=0.1
        )

    def run(self, trigger_event_class: Type[AbstractEvent]):
        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", "You are {agent_name}, {agent_description}."),
                (
                    "system",
                    "You are embedded in a simulated world with those properties {agent_world_state}",
                ),
                ("system", "Those are your goals: \n{goals}"),
                (
                    "system",
                    "And this is your current plan to achieve the goals: \n{plan}",
                ),
                (
                    "system",
                    "Here is your memories of all the events that you remember from being in this simulation: \n{memory}",
                ),
                (
                    "system",
                    "Those are the available entities that you can choose from: \n{available_entities}",
                ),
                (
                    "system",
                    "Here you have pre-filled parameters coming from your previous thoughts if any: \n{other_thoughts_filled_parameters}",
                ),
                (
                    "system",
                    "Here is the triggering event schema: \n{triggering_event_schema}",
                ),
                ("human", "{footer}"),
            ]
        )

        chain = create_structured_output_chain(
            output_schema=trigger_event_class.schema(),
            llm=self.llm,
            prompt=prompt,
            verbose=True,
        )
        response = chain.run(
            agent_name=self.agent_state.name,
            agent_description=self.agent_state.description,
            agent_world_state=self.agent_state.host_world_prompt,
            goals=self.agent_state.goals,
            plan=self.agent_state.plan,
            memory=self.agent_state.last_retrieved_memory,
            available_entities=self.agent_state.available_entities,
            other_thoughts_filled_parameters=self.agent_state.other_thoughts_filled_parameters,
            triggering_event_schema=json.dumps(trigger_event_class.schema()),
            footer="""Fill the parameters of the triggering event based on the previous context that you have about the world.
            """,
        )
        response = trigger_event_class.parse_obj(response)

        return response



from typing import List, Type
from genworlds.worlds.abstracts.world import AbstractWorld
from genworlds.worlds.abstracts.world_entity import AbstractWorldEntity
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.events.abstracts.action import AbstractAction

from genworlds.worlds.concrete.location_based.actions import (
    WorldSendsSameLocationEntities,
    WorldSendsSameLocationActionSchemas,
    WorldSetsAgentLocation,
)


class WorldLocationEntity(AbstractWorldEntity):
    location: str = None


class LocationWorld(AbstractWorld[WorldLocationEntity]):
    locations: list[str] = []

    def __init__(
        self,
        name: str,
        description: str,
        locations: list[str],
        agents: List[AbstractAgent] = [],
        objects: List[AbstractObject] = [],
        actions: List[Type[AbstractAction]] = [],
        id: str = None,
    ):
        self.locations = locations
        # availability = same location as sender id
        get_available_entities = WorldSendsSameLocationEntities(host_object=self)
        get_available_action_schemas = WorldSendsSameLocationActionSchemas(
            host_object=self
        )

        actions.append(get_available_entities)
        actions.append(get_available_action_schemas)
        actions.append(WorldSetsAgentLocation(host_object=self))

        super().__init__(
            name=name,
            description=description,
            agents=agents,
            objects=objects,
            actions=actions,
            get_available_entities=get_available_entities,
            get_available_action_schemas=get_available_action_schemas,
            id=id,
        )

    def add_location(self, location: str):
        self.locations.append(location)

    def remove_location(self, location: str):
        self.locations.remove(location)



from time import sleep
import threading
import websocket

from colorama import Fore


class SimulationSocketClient:
    def __init__(self) -> None:
        self.uri = "ws://127.0.0.1:7456/ws"
        self.websocket = websocket.WebSocketApp(
            self.uri,
            on_open=self.on_open,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
        )
        print(f"Connected to world socket server {self.uri}")

    def on_open(self, ws):
        print(f"World socket client opened connection to {self.uri}")

    def on_message(self, ws, message):
        thread_name = threading.current_thread().name
        print(f"{Fore.GREEN}[{thread_name}] World socket client received: {message}")

    def on_error(self, ws, error):
        print(f"World socket client error: {error}")

    def on_close(self, ws):
        print("World socket client closed connection")

    def send_messages_every_10secs(self):
        thread_name = threading.current_thread().name
        while True:
            sleep(10)
            message = "Hello World!"
            self.websocket.send(message)
            print(f"{Fore.CYAN}[{thread_name}] World socket client sent: {message}")


def main():
    world_socket_client = SimulationSocketClient()
    world_socket_client_2 = SimulationSocketClient()
    world_socket_client_3 = SimulationSocketClient()

    threading.Thread(
        target=world_socket_client.websocket.run_forever, name="Listener Test Thread"
    ).start()
    threading.Thread(
        target=world_socket_client_2.websocket.run_forever,
        name="Listener Test Thread 2",
    ).start()
    threading.Thread(
        target=world_socket_client_3.websocket.run_forever,
        name="Listener Test Thread 3",
    ).start()
    threading.Thread(
        target=world_socket_client.send_messages_every_10secs, name="Sender Test Thread"
    ).start()


main()



from __future__ import annotations
from uuid import uuid4
import threading
from typing import List
from genworlds.events.abstracts.action import AbstractAction
from genworlds.simulation.sockets.client import SimulationSocketClient
from genworlds.events.abstracts.event import AbstractEvent


class SimulationSocketEventHandler:
    def __init__(
        self,
        id: str,
        actions: List[AbstractAction] = [],
        external_event_classes: dict[str, AbstractEvent] = {},
        websocket_url: str = "ws://127.0.0.1:7456/ws",
    ):
        self.event_actions_dict: dict[str, AbstractAction] = {}
        self.id = id if id else str(uuid4())
        self.actions = actions
        for action in self.actions:
            self.register_action(action)

        self.simulation_socket_client = SimulationSocketClient(
            process_event=self.process_event, url=websocket_url
        )

    def register_action(self, action: AbstractAction):
        event_type = action.trigger_event_class.__fields__["event_type"].default
        if event_type not in self.event_actions_dict:
            self.event_actions_dict[event_type] = []
            self.event_actions_dict[event_type].append(action)
        else:
            self.event_actions_dict[event_type].append(action)

    def process_event(self, event):
        if event["event_type"] in self.event_actions_dict and (
            event["target_id"] == None or event["target_id"] == self.id
        ):
            # 0 bc the trigger_event_class is the same for all actions with the same event_type
            parsed_event = self.event_actions_dict[event["event_type"]][
                0
            ].trigger_event_class.parse_obj(event)

            for listener in self.event_actions_dict[event["event_type"]]:
                listener(parsed_event)

        if "*" in self.event_actions_dict:
            for listener in self.event_actions_dict["*"]:
                listener(event)

    def send_event(self, event: AbstractEvent):
        self.simulation_socket_client.send_message(event.json())

    def launch_websocket_thread(self):
        threading.Thread(
            target=self.simulation_socket_client.websocket.run_forever,
            name=f"{self.id} Thread",
            daemon=True,
        ).start()



from typing import List
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.agents.abstracts.thought import AbstractThought
from langchain.chat_models import ChatOpenAI
from enum import Enum
from pydantic import BaseModel, Field
from langchain.chains.openai_functions import (
    create_structured_output_chain,
)
from langchain.chat_models import ChatOpenAI
from langchain.prompts import ChatPromptTemplate


class ActionSchemaSelectorThought(AbstractThought):
    def __init__(
        self,
        agent_state: AbstractAgentState,
        openai_api_key: str,
        model_name: str = "gpt-3.5-turbo",
    ):
        self.agent_state = agent_state
        self.model_name = model_name
        self.llm = ChatOpenAI(
            model=self.model_name, openai_api_key=openai_api_key, temperature=0.1
        )

    def run(self):
        class PlanNextAction(BaseModel):
            """Plans for the next action to be executed by the agent."""

            action_name: str = Field(
                ...,
                description="Selects the action name of the next action to be executed from the list of available action names.",
            )
            is_action_valid: bool = Field(
                ..., description="Determines whether the next action is valid or not."
            )
            is_action_valid_reason: str = Field(
                ...,
                description="Then explains the rationale of whether it is valid or not valid action.",
            )
            new_plan: List[str] = Field(
                ..., description="The new plan to execute to achieve the goals."
            )

        action_schemas_full_string = "## Available Actions: \n\n"
        for (
            action_schema_key,
            action_schema_value,
        ) in self.agent_state.available_action_schemas.items():
            action_schemas_full_string += (
                "Action Name: "
                + action_schema_key
                + "\nAction Description: "
                + action_schema_value.split("|")[0]
                + "\n\n"
            )

        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", "You are {agent_name}, {agent_description}.\n"),
                (
                    "system",
                    "You are embedded in a simulated world with those properties {agent_world_state}\n",
                ),
                ("system", "Those are your goals: \n{goals}\n"),
                (
                    "system",
                    "And this is your current plan to achieve the goals: \n{plan}\n",
                ),
                (
                    "system",
                    "Here is your memories of all the events that you remember from being in this simulation: \n{memory}\n",
                ),
                (
                    "system",
                    "Those are the available actions that you can choose from: \n{available_actions}\n",
                ),
                ("human", "{footer}\n"),
            ]
        )

        chain = create_structured_output_chain(
            PlanNextAction.schema(), self.llm, prompt, verbose=True
        )

        response = chain.run(
            agent_name=self.agent_state.name,
            agent_description=self.agent_state.description,
            agent_world_state=self.agent_state.host_world_prompt,
            goals=self.agent_state.goals,
            plan=self.agent_state.plan,
            memory=self.agent_state.last_retrieved_memory,
            available_actions=action_schemas_full_string,
            footer="""Select the next action which must be a value of the available actions that you can choose from based on previous context.
Also select whether the action is valid or not, and if not, why.
And finally, state a new updated plan that you want to execute to achieve your goals. If your next action is going to sleep, then you don't need to state a new plan.
            """,
        )
        response = PlanNextAction.parse_obj(response)
        return response.action_name, response.new_plan



from typing import List, Optional
from genworlds.agents.abstracts.thought import AbstractThought
from genworlds.events.abstracts.action import AbstractAction
from genworlds.agents.concrete.basic_assistant.agent import BasicAssistant
from genworlds.agents.abstracts.agent_state import AbstractAgentState


def generate_basic_assistant(
    openai_api_key: str,
    agent_name: str,
    description: str,
    host_world_id: Optional[str] = None,
    initial_agent_state: Optional[AbstractAgentState] = None,
    other_thoughts: List[AbstractThought] = [],
    action_classes: List[type[AbstractAction]] = [],
    action_schema_chains: List[type[str]] = [],
    simulation_memory_persistent_path: str = "./",
):
    """
    Method for generating super simple dummy agents.
    """

    # Static initialized in the constructor

    if not initial_agent_state:
        initial_agent_state = AbstractAgentState(
            name=agent_name,
            id=agent_name,
            description=description,
            goals=[
                "Starts waiting and sleeps till the user starts a new question.",
                f"Once {agent_name} receives a user's question, he makes sure to have all the information before sending the answer to the user.",
                f"When {agent_name} has all the required information, he speaks to the user with the results through the agent_speaks_with_user_event.",
                "After sending the response, he waits for the next user question.",
                "If you have been waiting for any object or entity to send you an event for over 30 seconds, you will wait and sleep until you receive a new event.",
            ],
            plan=[],
            last_retrieved_memory="",
            other_thoughts_filled_parameters={},
            available_entities=[],
            available_action_schemas={},
            current_action_chain=[],
            host_world_prompt="",
            simulation_memory_persistent_path=simulation_memory_persistent_path,
            memory_ignored_event_types=(
                "world_sends_available_action_schemas_event",
                "world_sends_available_entities_event",
                "agent_wants_updated_state",
            ),
            wakeup_event_types=set(),  # fill
            is_asleep=False,
            action_schema_chains=action_schema_chains,
        )

    return BasicAssistant(
        openai_api_key=openai_api_key,
        name=agent_name,
        id=agent_name,
        description=description,
        host_world_id=host_world_id,
        initial_agent_state=initial_agent_state,
        action_classes=action_classes,
        other_thoughts=other_thoughts,
    )



import logging
import os
import colorlog


class LoggingFactory:
    colors = ["red", "yellow", "blue", "purple", "cyan", "green", "white"]
    color_index = 0
    loggers = {}

    @classmethod
    def get_logger(cls, name, level=None):
        # If a logger with this name already exists, return it
        if name in cls.loggers:
            return cls.loggers[name]

        # Create a handler for the logger
        handler = colorlog.StreamHandler()
        handler.setFormatter(
            colorlog.ColoredFormatter(
                # f'%(log_color)s%(levelname)-8s%(reset)s %({cls.colors[cls.color_index]})s[%(name)s] %(message)s'
                f"%({cls.colors[cls.color_index]})s[%(name)s] %(message)s"
            )
        )

        # Create a logger
        logger = colorlog.getLogger(name)
        logger.addHandler(handler)

        # Set the logging level
        if level == None:
            level = os.getenv("LOGGING_LEVEL", logging.INFO)
        logger.setLevel(level)
        logger.debug(f"Created logger {name} with level {logger.level}")

        # Cache the logger instance
        cls.loggers[name] = logger

        # Update color index for next logger
        cls.color_index = (cls.color_index + 1) % len(cls.colors)

        return logger



from abc import ABC, abstractmethod
from genworlds.agents.abstracts.agent_state import AbstractAgentState


class AbstractStateManager(ABC):
    state: AbstractAgentState

    @abstractmethod
    def get_updated_state(self) -> AbstractAgentState:
        """Retrieve the updated state"""
        pass



import json
from typing import List

from langchain import PromptTemplate, LLMChain
from langchain.chat_models import ChatOpenAI
import qdrant_client
from qdrant_client.http import models as rest
from langchain.vectorstores import Qdrant
from langchain.embeddings import OpenAIEmbeddings

from langchain.docstore.document import Document


class OneLineEventSummarizer:
    def __init__(self, openai_api_key: str, model_name: str = "gpt-3.5-turbo"):
        self.summary_template = """
        This is The last event coming from a web-socket, it is in JSON format:
        {event}
        Summarize what happened in one line.
        """

        self.summary_prompt = PromptTemplate(
            template=self.summary_template,
            input_variables=[
                "event",
            ],
        )

        self.chat = ChatOpenAI(
            temperature=0, model_name=model_name, openai_api_key=openai_api_key
        )
        self.chain = LLMChain(llm=self.chat, prompt=self.summary_prompt)

    def summarize(
        self,
        event: str,
    ) -> str:
        """
        Summarize the event in one line.
        """
        return self.chain.run(event=event)


class FullEventStreamSummarizer:
    def __init__(self, openai_api_key: str, model_name: str = "gpt-3.5-turbo"):
        self.small_summary_template = """
        This is the full event stream coming from a web-socket, it is in JSON format:
        {event_stream}
        Summarize what happened during the event stream in {k} paragraphs.
        
        SUMMARY:
        """

        self.summary_prompt = PromptTemplate(
            template=self.small_summary_template,
            input_variables=[
                "event_stream",
                "k",
            ],
        )

        self.chat = ChatOpenAI(
            temperature=0, model_name=model_name, openai_api_key=openai_api_key
        )
        self.small_summary_chain = LLMChain(llm=self.chat, prompt=self.summary_prompt)

    def summarize(
        self,
        event_stream: List[str],
        k: int = 5,
    ) -> str:
        """
        Summarize the event stream in k paragraphs.
        """
        if len(event_stream) <= 100:
            return self.small_summary_chain.run(event_stream=event_stream, k=k)
        else:
            # needs to be implemented
            return ""


class SimulationMemory:
    """
    Uses NMK Approach to summarize the event stream.
    """

    def __init__(
        self,
        openai_api_key: str,
        model_name: str = "gpt-3.5-turbo",
        n_of_last_events: int = 5,
        n_of_similar_events: int = 5,
        n_of_paragraphs_in_summary: int = 5,
    ):
        self.n_of_last_events = n_of_last_events  # last events
        self.n_of_similar_events = n_of_similar_events  # similar events
        self.n_of_paragraphs_in_summary = (
            n_of_paragraphs_in_summary  # paragraphs in the summary
        )
        self.full_summary = ""

        self.world_events = []
        self.summarized_events = []
        self.one_line_summarizer = OneLineEventSummarizer(
            openai_api_key=openai_api_key, model_name=model_name
        )
        self.full_event_stream_summarizer = FullEventStreamSummarizer(
            openai_api_key=openai_api_key, model_name=model_name
        )

        self.embeddings_model = OpenAIEmbeddings(openai_api_key=openai_api_key)

        client = qdrant_client.QdrantClient(location=":memory:")
        client.recreate_collection(
            collection_name="world-events",
            vectors_config={
                "content": rest.VectorParams(
                    distance=rest.Distance.COSINE,
                    size=1536,
                ),
            },
        )
        client.recreate_collection(
            collection_name="summarized-world-events",
            vectors_config={
                "content": rest.VectorParams(
                    distance=rest.Distance.COSINE,
                    size=1536,
                ),
            },
        )
        self.events_db = Qdrant(
            client=client,
            collection_name="world-events",
            embeddings=self.embeddings_model,
        )
        self.summarized_events_db = Qdrant(
            client=client,
            collection_name="summarized-world-events",
            embeddings=self.embeddings_model,
        )

    def add_event(self, event, summarize: bool = False):
        self.world_events.append(event)
        self.events_db.add_documents([Document(page_content=event)])
        if summarize:
            self._add_summarized_event(event)

    def _add_summarized_event(self, event):
        sum_event = self.one_line_summarizer.summarize(event)
        event_as_dict = json.loads(event)
        self.summarized_events.append(event_as_dict["created_at"] + " " + sum_event)
        self.summarized_events_db.add_documents([Document(page_content=sum_event)])

    def create_full_summary(self):
        self.full_summary = self.full_event_stream_summarizer.summarize(
            event_stream=self.world_events, k=self.n_of_paragraphs_in_summary
        )

    def _get_n_last_events(self, summarized: bool = False):
        if summarized:
            events = self.summarized_events[-self.n_of_last_events :]
        else:
            events = self.world_events[-self.n_of_last_events :]
        return events#[::-1]

    def _get_m_similar_events(self, query: str, summarized: bool = False):
        if self.n_of_similar_events < 1:
            return []

        if summarized:
            m_events = self.summarized_events_db.similarity_search(
                k=self.n_of_similar_events, query=query
            )
            return [el.page_content for el in m_events]
        else:
            m_events = self.events_db.similarity_search(
                k=self.n_of_similar_events, query=query
            )
            return [el.page_content for el in m_events]

    def get_event_stream_memories(self, query: str, summarized: bool = False):
        if len(self.world_events) <= self.n_of_last_events:
            last_events = self._get_n_last_events(summarized=summarized)
            nmk = (
                "\n\n# Your Memories\n\n"
                "## Last events from oldest to most recent\n\n" + "\n".join(last_events)
            )
            return nmk
        last_events = self._get_n_last_events(summarized=summarized)
        # similar_events = self._get_m_similar_events(query=query, summarized=summarized)
        nmk = (
            "\n\n# Your Memories\n\n"
            "## Full Summary\n\n" + self.full_summary
            # + "\n\n## Similar events\n\n"
            # + "\n".join(similar_events)
            + "\n\n## Last events from oldest to most recent\n\n"
            + "\n".join(last_events)
        )
        return nmk



from abc import ABC, abstractmethod
from typing import Type, Any, Dict
from enum import Enum
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.objects.abstracts.object import AbstractObject
from pydantic import BaseModel


class EntityTypeEnum(str, Enum):
    AGENT = "AGENT"
    OBJECT = "OBJECT"
    WORLD = "WORLD"


def get_entity_type(cls):
    from genworlds.worlds.abstracts.world import AbstractWorld

    if issubclass(cls, AbstractAgent):
        return EntityTypeEnum.AGENT
    elif issubclass(cls, AbstractWorld):
        return EntityTypeEnum.WORLD
    elif issubclass(cls, AbstractObject):
        return EntityTypeEnum.OBJECT
    else:
        return None


class AbstractWorldEntity(BaseModel, ABC):
    id: str
    entity_type: EntityTypeEnum
    entity_class: str
    name: str
    description: str

    class Config:
        extra = "allow"

    @classmethod
    def create(
        cls: Type["AbstractWorldEntity"],
        entity: AbstractObject,
        **additional_world_properties: Dict
    ) -> "AbstractWorldEntity":
        entity_cls = type(entity)
        entity_type = get_entity_type(entity_cls)
        entity_class = entity_cls.__name__
        id = entity.id
        name = entity.name
        description = entity.description
        # Combine predefined fields with any additional fields provided
        entity_data = {
            **{
                "id": id,
                "entity_type": entity_type,
                "entity_class": entity_class,
                "name": name,
                "description": description,
            },
            **additional_world_properties,
        }
        return cls(**entity_data)



import traceback
from time import sleep
import threading
from typing import List, Type

from genworlds.agents.utils.validate_action import validate_action
from genworlds.agents.abstracts.action_planner import AbstractActionPlanner
from genworlds.agents.abstracts.state_manager import AbstractStateManager
from genworlds.events.abstracts.action import AbstractAction
from genworlds.objects.abstracts.object import AbstractObject


class AbstractAgent(AbstractObject):
    """Abstract interface class for an Agent.

    This class represents an abstract agent that can think and perform actions
    within a simulation environment.
    """

    def __init__(
        self,
        name: str,
        id: str,
        description: str,
        state_manager: Type[AbstractStateManager],
        action_planner: Type[AbstractActionPlanner],
        host_world_id: str = None,
        actions: List[type[AbstractAction]] = [],
    ):
        self.action_planner = action_planner
        self.state_manager = state_manager
        super().__init__(name, id, description, host_world_id, actions)

    def think_n_do(self):
        """Continuously plans and executes actions based on the agent's state."""
        while True:
            try:
                if self.state_manager.state.is_asleep:
                    sleep(1)
                    continue
                else:
                    state = self.state_manager.get_updated_state()
                    action_schema, trigger_event = self.action_planner.plan_next_action(
                        state
                    )

                    if action_schema.startswith(self.id):
                        selected_action = [
                            action
                            for action in self.actions
                            if action.action_schema[0] == action_schema
                        ][0]
                        selected_action(trigger_event)
                    else:
                        self.send_event(trigger_event)
            except Exception as e:
                print(f"Error in think_n_do: {e}")
                traceback.print_exc()

    def launch(self):
        """Launches the agent by starting the websocket and thinking threads."""
        self.launch_websocket_thread()
        sleep(0.1)
        thinking_thread = threading.Thread(
            target=self.think_n_do,
            name=f"Agent {self.state_manager.state.id} Thinking Thread",
            daemon=True,
        )
        thinking_thread.start()



from abc import ABC, abstractmethod


class AbstractThought(ABC):
    @abstractmethod
    def run(self, llm_params: dict) -> str:
        """Run the brain with the given parameters and produce a response."""



import threading
from time import sleep
from genworlds.simulation.simulation import Simulation
from genworlds.simulation.sockets.server import start


def launch_simulation(simulation: Simulation):
    # Start the simulation socket in a parallel thread
    simulation_socket = threading.Thread(target=start)
    simulation_socket.start()

    # Wait for the socket to start
    sleep(1)

    # Launch the simulation
    simulation_thread = threading.Thread(target=simulation.launch)
    simulation_thread.start()



import os
import sys
import argparse
import threading
from typing import List
import logging

from fastapi import FastAPI, WebSocket, WebSocketDisconnect
import uvicorn

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class WebSocketManager:
    def __init__(self):
        self.active_connections: List[WebSocket] = []

    async def connect(self, websocket: WebSocket):
        await websocket.accept()
        self.active_connections.append(websocket)

    async def disconnect(self, websocket: WebSocket):
        self.active_connections.remove(websocket)

    async def send_update(self, data: str):
        closed_connections = []
        for connection in self.active_connections:
            try:
                await connection.send_text(data)
            except RuntimeError as e:
                if "Unexpected ASGI message" in str(e) and "websocket.close" in str(e):
                    closed_connections.append(connection)
                else:
                    raise e
        for closed_connection in closed_connections:
            self.active_connections.remove(closed_connection)


app = FastAPI()
websocket_manager = WebSocketManager()

socket_server_url = None


@app.on_event("shutdown")
async def shutdown_event():
    logger.info("SIGTERM received, stopping server...")
    sys.exit(0)


@app.websocket("/ws")
async def websocket_endpoint(websocket: WebSocket):
    await websocket_manager.connect(websocket)
    try:
        while True:
            data = await websocket.receive_text()
            logger.debug(f"Received data: {data}")
            print(data)
            await websocket_manager.send_update(data)
    except WebSocketDisconnect as e:
        logger.warning(f"WebSocketDisconnect: {e.code}")
    except Exception as e:
        logger.error(f"Exception: {type(e).__name__}, {e}", exc_info=True)
    finally:
        await websocket_manager.disconnect(websocket)


def start(host: str = "127.0.0.1", port: int = 7456, silent: bool = False):
    if silent:
        sys.stdout = open(os.devnull, "w")
        sys.stderr = open(os.devnull, "w")

    uvicorn.run(app, host=host, port=port, log_level="info")

    if silent:
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__


def start_thread(host: str = "127.0.0.1", port: int = 7456, silent: bool = False):
    threading.Thread(
        target=start,
        name=f"Websocket Server Thread",
        daemon=True,
        args=(
            host,
            port,
            silent,
        ),
    ).start()


def parse_args():
    parser = argparse.ArgumentParser(description="Start the world socket server.")
    parser.add_argument(
        "--port",
        type=int,
        help="The port to start the socket on.",
        default=7456,
        nargs="?",
    )
    parser.add_argument(
        "--host",
        type=str,
        help="The hostname of the socket.",
        default="127.0.0.1",
        nargs="?",
    )

    return parser.parse_args()


def start_from_command_line():
    args = parse_args()
    try:
        start(host=args.host, port=args.port)
    except BaseException as e:
        logger.error(e)
        sys.exit(0)


if __name__ == "__main__":
    start_from_command_line()



from abc import ABC, abstractmethod
from typing import List, Dict, Any, Tuple
from genworlds.agents.abstracts.thought import AbstractThought
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.events.abstracts.event import AbstractEvent


class AbstractActionPlanner(ABC):
    def __init__(
        self,
        action_schema_selector: AbstractThought,
        event_filler: AbstractThought,
        other_thoughts: List[
            AbstractThought
        ] = [],  # other trigger event param filler thoughts
    ):
        self.action_schema_selector = action_schema_selector
        self.event_filler = event_filler
        self.other_thoughts = other_thoughts

    def plan_next_action(self, state: AbstractAgentState) -> Tuple[str, AbstractEvent]:
        if len(state.current_action_chain) > 0:
            action_schema = state.current_action_chain.pop(0)
            trigger_event = self.fill_triggering_event(action_schema, state)
            return action_schema, trigger_event
        action_schema = self.select_next_action_schema(state)
        trigger_event = self.fill_triggering_event(action_schema, state)
        return action_schema, trigger_event

    @abstractmethod
    def select_next_action_schema(self, state: AbstractAgentState) -> str:
        """Select the next action schema based on the given state.

        :param state: The state based on which the next action schema should be selected.
        :return: The action schema, e.g., "MoveTo".
        """
        pass

    @abstractmethod
    def fill_triggering_event(
        self, next_action_schema: str, state: AbstractAgentState
    ) -> Dict[str, Any]:
        """Fill the triggering event parameters based on the given state.

        :param state: The state based on which the triggering event parameters should be filled.
        :return: The triggering event parameters, e.g., {"arg1": x, "arg2": y, ...}.
        """
        pass



from typing import Dict, List, Set, Any, Optional
from pydantic import BaseModel, Field
from genworlds.events.abstracts.event import AbstractEvent


class AbstractAgentState(BaseModel):
    """
    ### Serialize to JSON
    json_representation = instance.json()

    ### Deserialize from JSON
    instance = AbstractAgentState.parse_raw(json_representation)
    """

    # Static initialized in the constructor
    id: str = Field(..., description="Unique identifier of the agent.")
    description: str = Field(..., description="Description of the agent.")
    name: str = Field(..., description="Name of the agent.")
    host_world_prompt: str = Field(..., description="Prompt of the host world.")
    simulation_memory_persistent_path: Optional[str] = Field(
        None, description="Memory object storing the simulation data."
    )
    memory_ignored_event_types: Set[str] = Field(
        ...,
        description="Set of event types that will be ignored and not added to memory of the agent.",
    )
    wakeup_event_types: Set[str] = Field(
        ..., description="Events that can wake up the agent."
    )
    action_schema_chains: List[List[str]] = Field(
        ...,
        description="List of action schema chains that inhibit the action selector.",
    )
    goals: List[str] = Field(..., description="List of goals of the agent.")

    # Dynamically updated, so during one think_n_do cycle all of these must be updated somehow
    plan: List[str] = Field(..., description="List of actions that form the plan.")
    last_retrieved_memory: str = Field(
        ..., description="Last retrieved memory of the agent."
    )
    other_thoughts_filled_parameters: Dict[str, str] = Field(
        ..., description="Parameters filled by other thoughts."
    )
    available_action_schemas: Dict[str, Any] = Field(
        ..., description="Available action schemas with their descriptions."
    )
    available_entities: List[str] = Field(
        ..., description="List of available entities in the environment."
    )
    is_asleep: bool = Field(..., description="Indicates whether the agent is asleep.")
    current_action_chain: List[str] = Field(
        ..., description="List of action schemas that are currently being executed."
    )



from genworlds.objects.abstracts.object import AbstractObject
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.events.abstracts.action import AbstractAction


class AgentWantsUpdatedStateEvent(AbstractEvent):
    event_type = "agent_wants_updated_state"
    description = "Agent wants to update its state."
    # that gives available_action_schemas, and available_entities


class WorldSendsAvailableEntitiesEvent(AbstractEvent):
    event_type = "world_sends_available_entities_event"
    description = "Send available entities."
    available_entities: dict


class WorldSendsAvailableEntities(AbstractAction):
    trigger_event_class = AgentWantsUpdatedStateEvent
    description = "Send available entities."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentWantsUpdatedStateEvent):
        self.host_object.update_entities()
        all_entities = self.host_object.entities
        event = WorldSendsAvailableEntitiesEvent(
            sender_id=self.host_object.id,
            available_entities=all_entities,
            target_id=event.sender_id,
        )
        self.host_object.send_event(event)


class WorldSendsAvailableActionSchemasEvent(AbstractEvent):
    event_type = "world_sends_available_action_schemas_event"
    description = "The world sends the possible action schemas to all the agents."
    world_name: str
    world_description: str
    available_action_schemas: dict[str, str]


class WorldSendsAvailableActionSchemas(AbstractAction):
    trigger_event_class = AgentWantsUpdatedStateEvent
    description = "The world sends the possible action schemas to all the agents."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentWantsUpdatedStateEvent):
        self.host_object.update_action_schemas()
        self.host_object.update_entities()
        all_action_schemas = self.host_object.action_schemas
        all_entities = self.host_object.entities
        to_delete = []
        for action_schema in all_action_schemas:
            if all_entities[action_schema.split(":")[0]].entity_type == "AGENT" and action_schema.split(":")[0] != event.sender_id:
                to_delete.append(action_schema)

            if all_entities[action_schema.split(":")[0]].entity_type == "WORLD":
                to_delete.append(action_schema)
        
            if action_schema == f"{event.sender_id}:AgentListensEvents":
                to_delete.append(action_schema)

        for action_schema in to_delete:
            del all_action_schemas[action_schema]
            
        event = WorldSendsAvailableActionSchemasEvent(
            sender_id=self.host_object.id,
            world_name=self.host_object.name,
            world_description=self.host_object.description,
            available_action_schemas=all_action_schemas,
        )
        self.host_object.send_event(event)


class UserSpeaksWithAgentEvent(AbstractEvent):
    event_type = "user_speaks_with_agent_event"
    description = "The user speaks with an agent."
    message: str



import json
from datetime import datetime
from jsonschema import ValidationError, validate
from genworlds.agents.memories.simulation_memory import OneLineEventSummarizer


def validate_action(
    agent_id: str,
    action_schema: str,
    pre_filled_event: dict,
    available_action_schemas: dict,
):
    one_line_summarizer = OneLineEventSummarizer()  # missing key
    try:
        class_name, event_type = action_schema.split(":")
        trigger_event = {
            "event_type": event_type,
            "sender_id": agent_id,
            "created_at": datetime.now().isoformat(),
        }
        trigger_event.update(pre_filled_event)
        summary = one_line_summarizer.summarize(json.dumps(trigger_event))
        trigger_event["summary"] = summary

        event_schema = available_action_schemas[class_name][event_type]

        validate(trigger_event, event_schema)

        if class_name == "Self":
            is_my_action = True
        else:
            is_my_action = False
        return is_my_action, trigger_event
    except IndexError as e:
        return (
            f"Unknown command '{action_schema}'. "
            f"Please refer to the 'COMMANDS' list for available "
            f"commands and only respond in the specified JSON format."
        )
    except ValidationError as e:
        return (
            f"Validation Error in args: {str(e)}, pre_filled_event: {pre_filled_event}"
        )
    except Exception as e:
        return (
            f"Error: {str(e)}, {type(e).__name__}, pre_filled_event: {pre_filled_event}"
        )



from __future__ import annotations

from typing import Generic, TypeVar, List, Type
from time import sleep
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.worlds.abstracts.world_entity import AbstractWorldEntity

from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.events.abstracts.action import AbstractAction
from genworlds.simulation.sockets.server import start_thread as socket_server_start

WorldEntityType = TypeVar("WorldEntityType", bound=AbstractWorldEntity)


class AbstractWorld(Generic[WorldEntityType], AbstractObject):
    """
    An interface class representing a generic world in the simulation.
    """

    entities: dict[str, AbstractWorldEntity]
    action_schemas: dict[str, dict]

    def __init__(
        self,
        name: str,
        id: str,
        description: str,
        actions: List[Type[AbstractAction]],
        objects: List[AbstractObject],
        agents: List[AbstractAgent],
        get_available_entities: AbstractAction,
        get_available_action_schemas: AbstractAction,
    ):
        self.objects = objects
        self.agents = agents
        self.get_available_entities = get_available_entities
        self.get_available_action_schemas = get_available_action_schemas
        super().__init__(
            name=name, id=id, description=description, host_world_id=id, actions=actions
        )

    def update_entities(self):
        self.entities = {}
        self.entities[self.id] = self.get_entity_from_obj(self)
        for agent in self.agents:
            self.entities[agent.id] = self.get_entity_from_obj(agent)

        for obj in self.objects:
            self.entities[obj.id] = self.get_entity_from_obj(obj)

    def update_action_schemas(self):
        self.action_schemas = {}
        for action in self.actions:
            key, value = action.action_schema
            self.action_schemas[key] = value
        for obj in self.objects:
            for action in obj.actions:
                key, value = action.action_schema
                self.action_schemas[key] = value
        for agent in self.agents:
            for action in agent.actions:
                key, value = action.action_schema
                self.action_schemas[key] = value

    def get_entity_from_obj(self, obj: AbstractObject) -> WorldEntityType:
        """
        Returns the entity associated with the object.
        """
        return AbstractWorldEntity.create(obj)

    def get_entity_by_id(self, entity_id: str) -> AbstractWorldEntity:
        return self.entities[entity_id]

    def add_agent(self, agent: AbstractAgent):
        self.agents.append(agent)
        self.agents[-1].host_world_id = self.id
        self.agents[-1].launch()

    def add_object(self, obj: AbstractObject):
        self.objects.append(obj)
        self.objects[-1].host_world_id = self.id
        self.objects[-1].launch_websocket_thread()

    # TODO: delete objects and agents by id (stop thread, then remove from list)
    # TODO: update and restart objects and agents close threads and launch new ones
    # TODO: be able to stop the world and restart it

    def launch(self):
        socket_server_start()
        sleep(0.2)
        self.launch_websocket_thread()
        sleep(0.2)
        for agent in self.agents:
            sleep(0.1)
            self.add_agent(agent)

        for obj in self.objects:
            sleep(0.1)
            self.add_object(obj)



from typing import Dict, Tuple
from genworlds.events.abstracts.action import AbstractAction
from genworlds.agents.abstracts.thought import AbstractThought


class ThoughtAction(AbstractAction):
    """
    Abstract interface class for a Thought Action.

    This includes the list of thoughts required to fill the parameters of the trigger event.
    """

    # {parameter_name: [thought_class, run_dict]}
    required_thoughts: Dict[str, Tuple[AbstractThought, dict]]

    # in the __call__ method after executing the action you must clean the state
    # with self.host_object.state_manager.state.other_thoughts_filled_parameters = {}



from typing import List, Type
from genworlds.simulation.sockets.handlers.event_handler import (
    SimulationSocketEventHandler,
)
from genworlds.events.abstracts.action import AbstractAction


class AbstractObject(SimulationSocketEventHandler):
    """
    A Class representing a generic object in the simulation.
    """

    def __init__(
        self,
        name: str,
        id: str,
        description: str,
        host_world_id: str = None,
        actions: List[Type[AbstractAction]] = [],
    ):
        self.host_world_id = host_world_id
        self.name = name
        self.description = description

        super().__init__(id=id, actions=actions)



from __future__ import annotations
from abc import ABC, abstractmethod
import json
from typing import Any, Type, TypeVar, Generic, Tuple

from genworlds.events.abstracts.event import AbstractEvent

T = TypeVar("T", bound=AbstractEvent)


class AbstractAction(ABC, Generic[T]):
    trigger_event_class: Type[T]
    description: str

    def __init__(self, host_object: "AbstractObject"):
        self.host_object = host_object

    @property
    def action_schema(self) -> Tuple(str):
        """Returns the action schema as a string"""
        return (
            f"{self.host_object.id}:{self.__class__.__name__}",
            f"{self.description}|{self.trigger_event_class.__fields__['event_type'].default}|"
            + json.dumps(self.trigger_event_class.schema()),
        )
        # f"{type(self.host_object).__name__}|\n{self.host_object.description}|\n"

    @abstractmethod
    def __call__(self, event: T, *args: Any, **kwargs: Any) -> Any:
        """
        Execute the action, potentially generating one or more events.
        Send the events with self.socket_handler.send_event(event).

        :param event: The event that triggers the action.
        :param args: Additional positional arguments.
        :param kwargs: Additional keyword arguments.
        :return: The result of the action execution. The type of the return value can be any type
                 depending on the implementation.
        """
        pass



import json
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.events.abstracts.action import AbstractAction


class UserRequestsScreensToWorldEvent(AbstractEvent):
    event_type = "user_requests_screens_to_world"
    description = "The user requests the screens to the world."


class WorldSendsScreensToUserEvent(AbstractEvent):
    event_type = "world_sends_screens_to_user"
    description = "The world sends the screens to the user."
    screens_config: dict


class WorldSendsScreensToUser(AbstractAction):
    trigger_event_class = UserRequestsScreensToWorldEvent
    description = "The world sends the screens to the user."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: UserRequestsScreensToWorldEvent):
        with open(self.host_object.screens_config_path) as f:
            screens_config = json.load(f)

        event = WorldSendsScreensToUserEvent(
            sender_id=self.host_object.id,
            screens_config=screens_config,
        )
        self.host_object.send_event(event)



from typing import List, Type

from genworlds.events.abstracts.action import AbstractAction
from genworlds.worlds.abstracts.world import AbstractWorld
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.worlds.concrete.community_chat_interface.actions import (
    WorldSendsScreensToUser,
)
from genworlds.worlds.concrete.base.actions import (
    WorldSendsAvailableEntities,
    WorldSendsAvailableActionSchemas,
)


class ChatInterfaceWorld(AbstractWorld):
    def __init__(
        self,
        name: str,
        description: str,
        agents: List[AbstractAgent] = [],
        objects: List[AbstractObject] = [],
        actions: List[Type[AbstractAction]] = [],
        id: str = None,
        screens_config_path: str = "./screens_config.json",
    ):
        self.screens_config_path = screens_config_path
        # availability = all entities
        get_available_entities = WorldSendsAvailableEntities(host_object=self)
        get_available_action_schemas = WorldSendsAvailableActionSchemas(
            host_object=self
        )
        actions.append(get_available_entities)
        actions.append(get_available_action_schemas)
        actions.append(WorldSendsScreensToUser(host_object=self))

        super().__init__(
            name=name,
            description=description,
            agents=agents,
            objects=objects,
            actions=actions,
            get_available_entities=get_available_entities,
            get_available_action_schemas=get_available_action_schemas,
            id=id,
        )



from genworlds.events.abstracts.event import AbstractEvent
from genworlds.events.abstracts.action import AbstractAction
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.worlds.concrete.base.actions import (
    AgentGetsAvailableEntitiesEvent,
    WorldSendsAvailableEntitiesEvent,
    WorldSendsAvailableActionSchemasEvent,
)


class AgentMovesToNewLocation(AbstractEvent):
    event_type = "agent_moves_to_new_location"
    description = "Agent moves to a new location in the world."
    destination_location: str


class WorldSetsAgentLocationEvent(AbstractEvent):
    event_type = "world_sets_agent_location"
    description = "The new location has been set for the agent."


class WorldSetsAgentLocation(AbstractAction):
    trigger_event_class = AgentMovesToNewLocation

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentMovesToNewLocation):
        if event.destination_location not in self.host_object.locations:
            raise ValueError(
                f"Destination location {event.destination_location} is not in world locations {self.host_object.locations}"
            )
        self.host_object.get_entity_by_id(
            event.sender_id
        ).location = event.destination_location
        event = WorldSetsAgentLocationEvent(
            sender_id=self.host_object.id,
        )
        self.host_object.send_event(event)


class WorldSendsSameLocationEntities(AbstractAction):
    trigger_event_class = AgentGetsAvailableEntitiesEvent

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentGetsAvailableEntitiesEvent):
        self.host_object.update_entities()
        sender_entity = self.host_object.get_entity_by_id(event.sender_id)
        same_location_entities = {}
        for entity_id, entity in self.host_object.entities.items():
            if entity.location == sender_entity.location:
                same_location_entities[entity_id] = entity
        event = WorldSendsAvailableEntitiesEvent(
            sender_id=self.host_object.id,
            target_id=event.sender_id,
            available_entities=same_location_entities,
        )
        self.host_object.send_event(event)


class WorldSendsSameLocationActionSchemas(AbstractAction):
    trigger_event_class = AgentGetsAvailableEntitiesEvent  ## change that!!

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentGetsAvailableEntitiesEvent):
        self.host_object.update_action_schemas()
        sender_entity = self.host_object.get_entity_by_id(event.sender_id)
        sender_location = sender_entity.location
        same_location_action_schemas = {}
        for action_schema_id, action_schema in self.host_object.action_schemas.items():
            entity_id = action_schema_id.split(":")[0]
            entity_location = self.host_object.get_entity_by_id(entity_id).location
            if entity_location == sender_location:
                same_location_action_schemas[action_schema_id] = action_schema
        event = WorldSendsAvailableActionSchemasEvent(
            sender_id=self.host_object.id,
            target_id=event.sender_id,
            available_action_schemas=same_location_action_schemas,
        )
        self.host_object.send_event(event)



from typing import Dict, Any, List
import json
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.agents.abstracts.action_planner import AbstractActionPlanner
from genworlds.agents.abstracts.agent_state import AbstractAgentState
from genworlds.agents.abstracts.thought import AbstractThought
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.agents.concrete.basic_assistant.thoughts.action_schema_selector import (
    ActionSchemaSelectorThought,
)
from genworlds.agents.concrete.basic_assistant.thoughts.event_filler import (
    EventFillerThought,
)
from genworlds.agents.abstracts.thought_action import ThoughtAction
from genworlds.utils.schema_to_model import json_schema_to_pydantic_model


class BasicAssistantActionPlanner(AbstractActionPlanner):
    """This action planner selects the action schema with the highest priority and fills the event parameters with the highest priority."""

    def __init__(
        self,
        host_agent: AbstractAgent,
        openai_api_key,
        initial_agent_state: AbstractAgentState,
        other_thoughts: List[AbstractThought] = [],
    ):
        self.host_agent = host_agent
        action_schema_selector = ActionSchemaSelectorThought(
            openai_api_key=openai_api_key,
            agent_state=initial_agent_state,
            model_name="gpt-4",
        )
        event_filler = EventFillerThought(
            openai_api_key=openai_api_key,
            agent_state=initial_agent_state,
            model_name="gpt-4",
        )
        other_thoughts = other_thoughts
        super().__init__(
            action_schema_selector,
            event_filler,
            other_thoughts,
        )

    def select_next_action_schema(self, state: AbstractAgentState) -> str:
        # call action_schema_selector thought with the correct parameters
        (
            next_action_schema,
            updated_plan,
        ) = self.action_schema_selector.run()  # gives enum values
        state.plan = updated_plan
        if next_action_schema in [el[0] for el in state.action_schema_chains]:
            state.current_action_chain = state.action_schema_chains[
                [el[0] for el in state.action_schema_chains].index(next_action_schema)
            ][1:]
        return next_action_schema

    def fill_triggering_event(
        self, next_action_schema: str, state: AbstractAgentState
    ) -> Dict[str, Any]:
        # check if is a thought action and compute the missing parameters
        if next_action_schema.startswith(self.host_agent.id):
            next_action = [
                action
                for action in self.host_agent.actions
                if next_action_schema == action.action_schema[0]
            ][0]
            trigger_event_class = next_action.trigger_event_class
            if isinstance(next_action, ThoughtAction):
                for param in next_action.required_thoughts:
                    thought_class = next_action.required_thoughts[param]
                    thought = thought_class(self.host_agent.state_manager.state)
                    state.other_thoughts_filled_parameters[param] = thought.run()

        else:
            trigger_event_class_schema = json.loads(
                self.host_agent.state_manager.state.available_action_schemas[
                    next_action_schema
                ].split("|")[-1]
            )
            trigger_event_class = json_schema_to_pydantic_model(
                trigger_event_class_schema
            )

        trigger_event: AbstractEvent = self.event_filler.run(trigger_event_class)
        return trigger_event



import json
from genworlds.objects.abstracts.object import AbstractObject
from genworlds.events.abstracts.action import AbstractAction
from genworlds.events.abstracts.event import AbstractEvent
from genworlds.worlds.concrete.base.actions import (
    WorldSendsAvailableEntitiesEvent,
    WorldSendsAvailableActionSchemasEvent,
)


class UpdateAgentAvailableEntities(AbstractAction):
    trigger_event_class = WorldSendsAvailableEntitiesEvent
    description = "Update the available entities in the agent's state."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: WorldSendsAvailableEntitiesEvent):
        self.host_object.state_manager.state.available_entities = (
            event.available_entities
        )


class UpdateAgentAvailableActionSchemas(AbstractAction):
    trigger_event_class = WorldSendsAvailableActionSchemasEvent
    description = "Update the available action schemas in the agent's state."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: WorldSendsAvailableActionSchemasEvent):
        self.host_object.state_manager.state.available_action_schemas = (
            event.available_action_schemas
        )


class AgentWantsToSleepEvent(AbstractEvent):
    event_type = "agent_wants_to_sleep"
    description = "The agent wants to sleep. He has to wait for new events. The sender and the target ID is the agent's ID."


class AgentGoesToSleepEvent(AbstractEvent):
    event_type = "agent_goes_to_sleep"
    description = "The agent is waiting."


class AgentGoesToSleep(AbstractAction):
    trigger_event_class = AgentWantsToSleepEvent
    description = "The agent goes to sleep. He has to wait for new events. The sender and the target ID is the agent's ID."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentWantsToSleepEvent):
        self.host_object.state_manager.state.is_asleep = True
        self.host_object.send_event(
            AgentGoesToSleepEvent(sender_id=self.host_object.id, target_id=None)
        )
        print("Agent goes to sleep...")


class WildCardEvent(AbstractEvent):
    event_type = "*"
    description = "This event is used as a master listener for all events."


class AgentListensEvents(AbstractAction):
    trigger_event_class = WildCardEvent
    description = "The agent listens to all the events and stores them in his memory."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: dict):
        if event["target_id"] == self.host_object.id or event["target_id"] == None or event["sender_id"] == self.host_object.id:
            if (
                event["event_type"]
                not in self.host_object.state_manager.state.memory_ignored_event_types
            ):
                self.host_object.state_manager.memory.add_event(
                    json.dumps(event), summarize=True
                )  # takes time
            if (
                event["event_type"]
                in self.host_object.state_manager.state.wakeup_event_types
            ):
                self.host_object.state_manager.state.is_asleep = False
                print("Agent is waking up...")


class AgentSpeaksWithUserTriggerEvent(AbstractEvent):
    event_type = "agent_speaks_with_user_trigger_event"
    description = "An agent speaks with the user."
    message: str


class AgentSpeaksWithUserEvent(AbstractEvent):
    event_type = "agent_speaks_with_user_event"
    description = "An agent speaks with the user."
    message: str


class AgentSpeaksWithUser(AbstractAction):
    trigger_event_class = AgentSpeaksWithUserTriggerEvent
    description = "An agent speaks with the user."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentSpeaksWithUserTriggerEvent):
        self.host_object.send_event(
            AgentSpeaksWithUserEvent(
                sender_id=self.host_object.id,
                target_id=event.target_id,
                message=event.message,
            )
        )


class AgentSpeaksWithAgentEvent(AbstractEvent):
    event_type = "agent_speaks_with_agent_event"
    description = "An agent speaks with another agent."
    message: str


class AgentSpeaksWithAgent(AbstractAction):
    trigger_event_class = AgentSpeaksWithAgentEvent
    description = "An agent speaks with another agent."

    def __init__(self, host_object: AbstractObject):
        super().__init__(host_object=host_object)

    def __call__(self, event: AgentSpeaksWithAgentEvent):
        self.host_object.send_event(event)



import threading
from genworlds.simulation.sockets.client import SimulationSocketClient


class TestUser:
    def __init__(
        self,
    ):
        self.id = "test_user"
        self.name = "Test User"
        self.description = "A test user for the simulation."
        self.socket_client = SimulationSocketClient(process_event=lambda x: "")
        threading.Thread(
            target=self.socket_client.websocket.run_forever,
            name=f"{self.id} Thread",
            daemon=True,
        ).start()



from __future__ import annotations

import threading
from uuid import uuid4
from typing import List
import time

from genworlds.objects.abstracts.object import AbstractObject
from genworlds.agents.abstracts.agent import AbstractAgent
from genworlds.worlds.abstracts.world import AbstractWorld


class Simulation:
    def __init__(
        self,
        name: str,
        description: str,
        world: AbstractWorld,
        objects: List[tuple[AbstractObject, dict]],
        agents: List[tuple[AbstractAgent, dict]],
        stop_event: threading.Event = None,
    ):
        self.id = str(uuid4())
        self.name = name
        self.description = description
        self.world = world
        self.objects = objects
        self.agents = agents
        self.stop_event = stop_event

    def add_agent(self, agent: AbstractAgent, **world_properties):
        self.agents.append([agent, world_properties])
        self.agents[-1][0].world_spawned_id = self.world.id
        self.world.add_agent(self.agents[-1][0], **self.agents[-1][1])
        self.agents[-1][0].launch()

    def add_object(self, obj: AbstractObject, **world_properties):
        self.objects.append([obj, world_properties])
        self.objects[-1][0].world_spawned_id = self.world.id
        self.world.add_object(self.objects[-1][0], **self.objects[-1][1])
        self.objects[-1][0].launch_websocket_thread()

    # TODO: delete objects and agents
    # TODO: update and restart objects and agents

    def launch(self):
        # Register agents and objects with the world
        for agent, world_properties in self.agents:
            agent.world_spawned_id = self.world.id
            self.world.register_agent(agent, **world_properties)

        for obj, world_properties in self.objects:
            obj.world_spawned_id = self.world.id
            self.world.register_object(obj, **world_properties)

        # Launch the world
        self.world.launch_websocket_thread()

        time.sleep(1)

        for agent, world_properties in self.agents:
            time.sleep(0.1)
            agent.launch()

        for obj, world_properties in self.objects:
            time.sleep(0.1)
            obj.launch_websocket_thread()

        # Make the application terminate gracefully
        while True:
            # TODO: pass the stop event to the world and the objects and the agents
            if self.stop_event and self.stop_event.is_set():
                break

            try:
                time.sleep(1)
            except KeyboardInterrupt:
                break
